/*
 * pi - P user Interface
 * 
 * $Id: tag.h,v 1.1.1.1 2001/02/13 18:30:27 malekith Exp $
 * Author: Michal Moskal <malekith@pld.org.pl>
 * include COPYING-GNU
 */

#ifndef _PI_TAGS_H
#define _PI_TAGS_H

#include <stdarg.h>

/* used tags ... */
enum {
/* marks end of tags, simply 0 */
pi_tag_end,		/* int */
/* these are used only for init */
pi_tag_x,
pi_tag_y,
pi_tag_w,
pi_tag_h,
/* supertype, like Button for BeepingButton: */
pi_tag_super,		/* struct pi_object* */
/* copied to widget->flags */
pi_tag_flags,		/* unsigned long */
/* event handlers */
pi_tag_key_handler,	/* int (*)(struct pi_object *, int), ret 1 if handled */
pi_tag_prekey_handler,	/* same, executed *before* ctx->active, can be used
			 * to intercept events */
pi_tag_cc_handler,	/* void (*)(struct pi_object *, int) */
pi_tag_draw_handler,	/* void (*)(struct pi_object *) */
pi_tag_kill_handler,	/* void (*)(struct pi_object *) */

pi_tag_cache1,
pi_tag_cache2,
pi_tag_cache3,
pi_tag_cache4,

pi_tag_userdata1,
pi_tag_userdata2,
pi_tag_userdata3,
pi_tag_userdata4,

/* widget specific tags */
pi_tag_caption,		/* char* */
pi_tag_modal_result,	/* int */
pi_tag_cc,

/* progressbar : */
pi_tag_min,
pi_tag_max,
pi_tag_pos,

/* root: */
pi_tag_text_left_up,
pi_tag_text_right_up,
pi_tag_text_middle_up,
pi_tag_text_left_down,
pi_tag_text_right_down,
pi_tag_text_middle_down,

/* scrollable: */
pi_tag_draw_line,
pi_tag_top,
pi_tag_menu_like,
/* pi_tag_max, */
/* pi_tag_pos, */

/* msgbox: */
pi_tag_button_ok,
pi_tag_button_yes,
pi_tag_button_no,
pi_tag_title,
/* pi_tag_caption, */

/* colors */
pi_tag_color_norm,
pi_tag_color_norm_hl,
pi_tag_color_acct,
pi_tag_color_acct_hl,
pi_tag_color_spec,
pi_tag_color_spec_hl,

pi_tag_internal = 0x1000,
pi_tag_user = 0x2000,
};

#define pi_tag_id(tag) ((tag) & 0xffff)
#define pi_tag_flags(tag) ((tag) & 0xffff0000)

/* tag flags */
enum {
/* tag data is pointer to function returing requested information,
 * e.g. for pi_tag_x it could be:
 * 	static long getx(struct pi_object *ctx)
 * so it's of type 
 * 	long (*func)(struct pi_taglist *)
 * or
 * 	void* (*func)(struct pi_taglist *)
 * *always* use long never int !
 */
pi_tf_func = 0x00010000,
/* when this is set void*data is free()ed */
pi_tf_free = 0x00020000,
};

/* unsigned long used here to get proper align on 64 bit archs 
 * to get int from pi_tag *ptr use:
 * (long)(ptr->data)
 * it's important *not* to use (int) , it would result in
 * possible problems (if sizeof(long) > sizeof(int) && big_endian)
 */
struct pi_tag {
	unsigned long tag;
	void *data;
};

enum {
pi_f_can_get_focus = 0x00000001,	/* whatever can be made active */
};

struct pi_object {
	int allocated;		/* number of entries allocated in *list */
	int used;		/* number of used entries including NULL */
	struct pi_object 
		*next, 		/* next widget in given parent */
		*prev, 		/* go figure :) */
		*parent, 	/* parent widget (button->window->screen) */
		*children, 	/* like sub forms, buttons etc */
		*last_child,
		*active;	/* active widget (highlighted button etc) */
	int x, y, w, h;
	unsigned long flags;
	struct pi_widget *widget;
	struct pi_tag *tags;
};

void *pi_fetch_data(struct pi_object *ctx, unsigned long tag, void *def);
int pi_exists(struct pi_object *ctx, unsigned long tag);
void pi_set_tag(struct pi_object *ctx, unsigned long tag, ...);
struct pi_object *pi_add_object(struct pi_object *ctx, ...);
struct pi_object *pi_add_objectv(struct pi_object *ctx, va_list ap);
void pi_kill_object(struct pi_object *ctx);

#define pi_intd(name, def) ((long)pi_fetch_data(ctx, pi_tag_ ## name, (void*)def))
#define pi_ptrd(name, def) (pi_fetch_data(ctx, pi_tag_ ## name, def))
#define pi_int(name) ((long)pi_fetch_data(ctx, pi_tag_ ## name, 0))
#define pi_ptr(name) (pi_fetch_data(ctx, pi_tag_ ## name, 0))

#define pi_ointd(ctx, name, def) ((long)pi_fetch_data(ctx, pi_tag_ ## name, (void*)def))
#define pi_optrd(ctx, name, def) (pi_fetch_data(ctx, pi_tag_ ## name, def))
#define pi_oint(ctx, name) ((long)pi_fetch_data(ctx, pi_tag_ ## name, 0))
#define pi_optr(ctx, name) (pi_fetch_data(ctx, pi_tag_ ## name, 0))

#endif /* _PI_TAGS_H */
