/*
 * pi - P user Interface
 * 
 * $Id: scrollable.c,v 1.1.1.1 2001/02/13 18:30:27 malekith Exp $
 * Author: Michal Moskal <malekith@pld.org.pl>
 * include COPYING-GNU
 */

#include <pi/pi.h>
#include <string.h>
#include <stdio.h>

/*
 * this is called from pi_scrollable_draw(). line_no is line to draw,
 * w is width we can print in. hl is 1 if it's highlighted position
 * in menulike scrollables. color is initially set to color_norm(hl?_hl:).
 * cursor is positioned at right place.
 * if line_no == -1 then sth like "~" should be printed.
 */
void pi_scrollable_draw_line(struct pi_object *ctx, int line_no, int w, int hl)
{
	char buf[100];
	(void)ctx;
	if (line_no == -1)
		strcpy(buf, "~");
	else
		sprintf(buf, "%c %d w=%d", hl?'*':' ', line_no, w);
	pi_addstrn_fill(buf, w);
}

void pi_scrollable_draw(struct pi_object *ctx)
{
	int c1, c2, ch1, ch2;
	int l1, l2;
	int x, y, n;
	int line, top, max, cur_pos = -1;
	int menu_like;
	void (*draw_line)(struct pi_object *, int, int, int);

	draw_line = pi_ptr(draw_line);
	
	pi_locate(ctx, &x, &y);
	
	c1 = pi_int(color_norm);
	c2 = pi_int(color_norm_hl);
	ch1 = pi_int(color_spec);
	ch2 = pi_int(color_spec_hl);

	max = pi_int(max);
	/* range control */
	top = pi_int(top);
	if (top > max - ctx->h)
		top = max - ctx->h;
	if (top < 0)
		top = 0;
	if ((menu_like = pi_int(menu_like))) {
		cur_pos = pi_int(pos);
		if (cur_pos < 0)
			cur_pos = 0;
		if (cur_pos >= max)
			cur_pos = max - 1;
		if (cur_pos < top) 
			top = cur_pos;
		if (cur_pos > ctx->h + top - 1) 
			top = cur_pos - ctx->h + 1;
		pi_set_tag(ctx, pi_tag_pos, cur_pos);
	}
	pi_set_tag(ctx, pi_tag_top, top);

	/* let's draw it ! */
	n = ctx->h;
	line = top;
	/* well, this seems to look like voodoo but it works :) */
	l1 = -((top + 1) * ctx->h) / max - 1 + ctx->h;
	l2 = -(top - 1 + ctx->h) * ctx->h / max - 1 + ctx->h;
	while (n--) {
		pi_goto(x, y);
		pi_color((line == cur_pos) ? c2 : c1);
		draw_line(ctx, line >= max ? -1 : line, 
		          ctx->w - 1, line == cur_pos);
		pi_goto(x + ctx->w - 1, y);

		if (n > l1 || n < l2) {
			pi_color(ch1);
			pi_putch(':');
		} else {
			pi_color(ch2);
			pi_putch('#');
		}
		y++;
		line++;
	}
}

int pi_scrollable_key(struct pi_object *ctx, int key)
{
	int delta = 0;
	int menu_like;

	menu_like = pi_int(menu_like);
	
	if (menu_like && (key == ' ' || key == '\n')) {
		if (pi_int(cc))
			pi_send_cc(ctx, pi_int(cc));
		return 1;
	} else 
		switch (key) {
		case '-':
		case pi_key_up:
			delta = -1;
			break;
		case '+':
		case pi_key_down:
			delta = +1;
			break;
		case pi_key_pgup:
			delta = -ctx->h;
			break;
		case pi_key_pgdn:
			delta = +ctx->h;
			break;
		default:
			return 0;
		}
		
	if (menu_like) 
		pi_set_tag(ctx, pi_tag_pos, pi_int(pos) + delta);
	else 
		pi_set_tag(ctx, pi_tag_top, pi_int(top) + delta);
	pi_draw(ctx);
	
	return 1;
}

struct pi_object *pi_new_scrollable(struct pi_object *ctx, struct pi_object *super)
{
	pi_new_init((0,
			pi_tag_flags, pi_f_can_get_focus,
			pi_tag_key_handler, pi_scrollable_key,
			pi_tag_draw_handler, pi_scrollable_draw,
			pi_tag_cc, 0,
			pi_tag_menu_like, 0,
			pi_tag_draw_line, pi_scrollable_draw_line,
			PI_COLOR(norm, lightgray, blue),
			PI_COLOR(norm_hl, black, cyan),
			PI_COLOR(spec, blue, blue),
			PI_COLOR(spec_hl, green, green),
	0), object);

	pi_init_size(ret, 0, 0, 20, 10);

	pi_copy_int(cc);

	return ret;
}
